package com.zanxiang.game.back.serve.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.sd4324530.jtuple.Tuple2;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAccountIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAppIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentAppApiRoleRegisterMapper;
import com.zanxiang.game.back.serve.pojo.entity.*;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.service.*;
import com.zanxiang.module.util.encryption.Md5Util;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class GameTencentAppApiRoleRegisterServiceImpl extends ServiceImpl<GameTencentAppApiRoleRegisterMapper, GameTencentAppApiRoleRegister>
implements IGameTencentAppApiRoleRegisterService {

    @Autowired
    private IGameTencentAppApiBackLogService gameTencentAppApiBackLogService;
    @Autowired
    private IGameTencentAppApiUserService gameTencentAppApiUserService;
    @Autowired
    private IGameTencentAppCallbackService gameTencentAppCallbackService;
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean roleRegisterBack(GameTencentAppApiRoleRegister roleRegisterLog) {
        Tuple2<BackStatusEnum, String> backStatus = doCallback(roleRegisterLog);
        return update(new LambdaUpdateWrapper<GameTencentAppApiRoleRegister>()
                .set(GameTencentAppApiRoleRegister::getIsBack, backStatus.first.getBackStatus())
                .set(GameTencentAppApiRoleRegister::getBackLog, backStatus.second)
                .eq(GameTencentAppApiRoleRegister::getId, roleRegisterLog.getId())
        );
    }

    @Override
    public boolean tencentUserReport(List<Long> ids) {
        listByIds(ids).forEach(roleRegisterLog -> {
            Tuple2<BackStatusEnum, String> backStatus = doCallback(roleRegisterLog);
            update(new LambdaUpdateWrapper<GameTencentAppApiRoleRegister>()
                    .set(GameTencentAppApiRoleRegister::getIsBack, backStatus.first.getBackStatus())
                    .set(GameTencentAppApiRoleRegister::getBackLog, backStatus.second)
                    .eq(GameTencentAppApiRoleRegister::getId, roleRegisterLog.getId())
            );
        });
        return true;
    }

    private Tuple2<BackStatusEnum, String> doCallback(GameTencentAppApiRoleRegister roleRegisterLog) {
        GameTencentAppApiUser user = gameTencentAppApiUserService.getOne(new LambdaQueryWrapper<GameTencentAppApiUser>()
                .eq(GameTencentAppApiUser::getGameId, roleRegisterLog.getGameId())
                .eq(GameTencentAppApiUser::getUserId, roleRegisterLog.getUserId())
                .eq(GameTencentAppApiUser::getAdAccountId, roleRegisterLog.getAdAccountId())
                .orderByDesc(GameTencentAppApiUser::getCreateTime)
                .last("limit 1")
        );
        if (user != null) {
            if (user.getCallbackId() == null || user.getCallbackId() < 1) {
                return Tuple2.with(BackStatusEnum.FAILED, "用户匹配不到广告");
            }
            if (BackStatusEnum.NO.getBackStatus().equals(user.getIsBack())) {
                gameTencentAppApiUserService.userBack(user, true);
            } else {
                // 注册已回传，则默认认为创角也回传了
                return Tuple2.with(BackStatusEnum.NO, "用户第二+次创角不回传");
            }
        } else  {
            return Tuple2.with(BackStatusEnum.FAILED, "找不到用户");
        }

        GameTencentAppCallback callback = gameTencentAppCallbackService.getById(user.getCallbackId());

        DataReportOfAccountIdRpcDTO dataReportOfAccountIdRpcDTO = DataReportOfAccountIdRpcDTO.builder()
                .accountId(user.getAdAccountId())
                .userActionSetId(user.getUserActionSetId())
                .action(UserActionRpcDTO.builder()
                        .actionTime(roleRegisterLog.getRegisterTime())
                        .actionType(ActionTypeEnum.CREATE_ROLE.getActionType())
                        .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                .hashImei(StringUtils.isBlank(user.getImei()) ? null : Md5Util.encrypt32(user.getImei().toLowerCase()).toLowerCase())
                                .hashIdfa(StringUtils.isBlank(user.getOaid()) ? null : Md5Util.encrypt32(user.getIdfa().toLowerCase()).toLowerCase())
                                .hashOaid(StringUtils.isBlank(user.getOaid()) ? null : Md5Util.encrypt32(user.getOaid().toLowerCase()).toLowerCase())
                                .build())
                        .trace(UserActionRpcDTO.TraceRpcDTO.builder()
                                .clickId(callback.getClickId())
                                .build())
                        .build())
                .build();

        GameTencentAppApiBackLog gameTencentBackLog = GameTencentAppApiBackLog.builder()
                .gameId(roleRegisterLog.getGameId())
                .userId(roleRegisterLog.getUserId())
                .adAccountId(roleRegisterLog.getAdAccountId())
                .actionTime(roleRegisterLog.getRegisterTime())
                .createTime(LocalDateTime.now())
                .actionType(ActionTypeEnum.CREATE_ROLE.getActionType())
                .imei(roleRegisterLog.getImei())
                .oaid(roleRegisterLog.getOaid())
                .androidId(roleRegisterLog.getAndroidId())
                .idfa(roleRegisterLog.getIdfa())
                .caid(roleRegisterLog.getCaid())
                .build();
        ResultVO<Boolean> result = userActionSetRpc.reportByAccountId(dataReportOfAccountIdRpcDTO);
        BackStatusEnum backStatus = BackStatusEnum.FAILED;
        if (result.isSuccess()) {
            backStatus = BackStatusEnum.SUCCESS;
            gameTencentBackLog.setBackLog("回传成功");
        } else {
            gameTencentBackLog.setBackLog(result.getMsg());
        }
        gameTencentAppApiBackLogService.save(gameTencentBackLog);
        return Tuple2.with(backStatus, gameTencentBackLog.getBackLog());
    }
}
